using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Text;
using System.IO;
using System.Text;
using System.Windows.Forms;
using System.Xml;

//using System.Diagnostics;

namespace Titulkovac
{
	public partial class Titulkovac : Form
	{
		public string sVersion = "02.01";

		public Font fFont;
		public int iFontSize;
		public Color cColorMessage;
		public Color cColorBackground;
		public int iDelay;
		public int iMovement;
		public string sSeparator;
		public string sMessage;
		public bool bMessageRunning = true;
		protected Timer tTimer;
		protected char[] chMessage;
		protected int iMessagePoint;
		protected XmlDocument xmlProperties = new XmlDocument();

		public Titulkovac()
		{
			getXMLProperties();
			InitializeComponent();
			setFormProperties();
			startMessageTimmer();
		}

		private void editaceToolStripMenuItem_Click(object sender, EventArgs e)
		{
			this.bMessageRunning = false;
			frmEdit frmE = new frmEdit();
			frmE.cmbFont.Text = this.fFont.Name;
			frmE.txtFontSize.Text = this.fFont.Size.ToString();
			frmE.cmbColorMessage.Text = this.cColorMessage.Name;
			frmE.cmbColorBackground.Text = this.cColorBackground.Name;
			frmE.txtDelay.Text = this.iDelay.ToString();
			frmE.txtMovement.Text = this.iMovement.ToString();
			frmE.txtSeparator.Text = this.sSeparator;
			frmE.txtMessage.Text = this.sMessage;
			frmE.ShowDialog(this);
			if (frmE.DialogResult == DialogResult.Cancel) {
				//zruseno
			}
			else if (frmE.DialogResult == DialogResult.OK) {
				setXmlFileProperties(frmE.cmbFont.Text, frmE.txtFontSize.Text, frmE.txtMessage.Text, frmE.txtSeparator.Text,
					frmE.cmbColorMessage.Text, frmE.cmbColorBackground.Text, frmE.txtDelay.Text, frmE.txtMovement.Text);
				getXMLProperties();
				setFormProperties();
			}
			else {
				//chybne tlacitko???
			}
			startMessageTimmer();
			this.bMessageRunning = true;
		}

		private void infoToolStripMenuItem_Click(object sender, EventArgs e)
		{
			frmInfo frmInf = new frmInfo();
			frmInf.txtVersionNumber.Text = this.sVersion;
			frmInf.ShowDialog(this);
		}

		private void konecToolStripMenuItem_Click(object sender, EventArgs e)
		{
			this.Close();
		}

		//nastaveni fontu
		private void setFont(string sFont, string sFontSize) {
			this.fFont = new Font(sFont, Int32.Parse(sFontSize));
		}
		//nastaveni barvy pozadi
		private void setColorBackground(string sColorBackground) {
			this.cColorBackground = System.Drawing.Color.FromName(sColorBackground);
		}
		//nastaveni barvy textu
		private void setColorMessage(string sColorMessage) {
			this.cColorMessage = System.Drawing.Color.FromName(sColorMessage);
		}
		//nastaveni zpozdeni
		private void setDelay(string sDelay) {
			this.iDelay = Int32.Parse(sDelay);
		}
		//nastaveni odsazeni od spodniho okraje
		private void setMovement(string sMovement) {
			this.iMovement = Int32.Parse(sMovement);
		}
		//nastaveni oddelovace zpravy
		private void setSeparator(string sSeparator) {
			this.sSeparator = sSeparator;
		}
		//nastaveni vlastni zpravy
		private void setMessage(string sMessage) {
			this.chMessage = null;
			this.iMessagePoint = 0;
			this.sMessage = sMessage;
		}

		//nastaveni pole s textem zpravy
		private void setMessageBuffer() {
			string s = this.sMessage + this.sSeparator;
			this.chMessage = new char[s.Length];
			for (int i = 0; i < s.Length; i++) {
				this.chMessage[i] = Char.Parse(s.Substring(i, 1));
			}
		}
		//nastaveni vlastnosti formulare
		private void setFormProperties() {
			this.Width = Screen.PrimaryScreen.WorkingArea.Width;
			this.Height = Int32.Parse(this.fFont.Size.ToString()) + Int32.Parse(this.fFont.Size.ToString())/2 + 3;
			this.Location = new Point(0, (Screen.PrimaryScreen.WorkingArea.Height - this.Height)+(-1*iMovement));
			this.BackColor = this.cColorBackground;
			try {
				this.txtMessage.BackColor = this.cColorBackground;
				this.txtMessage.ForeColor = this.cColorMessage;
				this.txtMessage.Font = this.fFont;
				this.txtMessage.Height = Int32.Parse(this.fFont.Size.ToString()) + Int32.Parse(this.fFont.Size.ToString()) / 2;
				this.txtMessage.Width = this.Width - this.Width / 100;
			}
			catch (Exception ex) {
				//Debug.WriteLine("Chyba: " + ex.StackTrace);
				MessageBox.Show("Chyba: " + ex.StackTrace);
			}
		}

		//spusteni casovace pro posun zobrazene zpravy
		private void startMessageTimmer() {
			try {
				tTimer.Tick -= new EventHandler(showMessage);
				tTimer = null;
			}
			catch (Exception ex) {
				//MessageBox.Show("Chyba: " + ex.StackTrace);
			}
			tTimer=new Timer();
			tTimer.Interval = this.iDelay+1;
			tTimer.Enabled = true;
			tTimer.Tick += new EventHandler(showMessage);
		}

		//vlastni zobrazeni zpravy volane z casovace
		private void showMessage(object sender, EventArgs e) {
			if (this.bMessageRunning) {
				if (this.chMessage == null) {
					setMessageBuffer();
					this.txtMessage.Text = null;
				}
				if (iMessagePoint >= this.chMessage.Length) {
					this.iMessagePoint = 0;
				}
				if (this.txtMessage.Text == null || this.txtMessage.Text == "") {
					StringBuilder sb = new StringBuilder();
					while (sb.Length < this.txtMessage.Width/(this.fFont.Size/2)) {
						sb.Append(" ");
					}
					this.txtMessage.Text = sb.ToString();
				}
				else {
					this.txtMessage.Text = this.txtMessage.Text.Substring(1, this.txtMessage.Text.Length - 1) + this.chMessage[iMessagePoint++];
				}
			}
			else {
				//nedelam nic
			}
		}

		//nacteni vlastnosti aplikace z XML souboru
		private void getXMLProperties() {
			//pokud XML soubor neexistuje, vytvori se a naplni defaultnimi hodnotami
			if (!File.Exists(System.IO.Path.Combine(Application.StartupPath, "Titulkovac.xml"))) {
				createNewXmlFileProperties();
				setXmlFileProperties("Times New Roman", "30",
					"Zde je njak pokusn text, aby bylo vidt, jak se vechno zobraz:         , + = - * / 1 2 3 4 5 6 7 8 9 0 ! @ # $ %       ",
					" * * * ", "Yellow", "Black", "200", "0");
			}
			try {
				xmlProperties.Load(System.IO.Path.Combine(Application.StartupPath, "Titulkovac.xml"));
				setFont(xmlProperties.SelectSingleNode("/titulkovac/font").Attributes.GetNamedItem("type").InnerText.ToString(),
					xmlProperties.SelectSingleNode("/titulkovac/font").Attributes.GetNamedItem("size").InnerText.ToString());
				setColorMessage(xmlProperties.SelectSingleNode("/titulkovac/message").Attributes.GetNamedItem("color-text").InnerText.ToString());
				setColorBackground(xmlProperties.SelectSingleNode("/titulkovac/message").Attributes.GetNamedItem("color-background").InnerText.ToString());
				setSeparator(xmlProperties.SelectSingleNode("/titulkovac/message").Attributes.GetNamedItem("separator").InnerText.ToString());
				//MessageBox.Show("Mess: "+xmlProperties.SelectSingleNode("/titulkovac/message").InnerText.ToString());
				setMessage(xmlProperties.SelectSingleNode("/titulkovac/message").InnerText.ToString());
				setDelay(xmlProperties.SelectSingleNode("/titulkovac/application").Attributes.GetNamedItem("delay").InnerText.ToString());
				setMovement(xmlProperties.SelectSingleNode("/titulkovac/application").Attributes.GetNamedItem("movement").InnerText.ToString());
			}
			catch (Exception ex) {
				MessageBox.Show("Chyba pri nacitani dat: " + ex.StackTrace, "Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
			}
		}

		//nastaveni (zmena) vlastnosti do XML souboru
		private void setXmlFileProperties(string sFnt, string sFntSize, string sMsg, string sMsgSeparator,
				string sMsgColorText, string sMsgColorBackground, string sDel, string sMove) {
			xmlProperties.SelectSingleNode("/titulkovac/font").Attributes.GetNamedItem("type").InnerText = sFnt;
			//MessageBox.Show(xmlProperties.SelectSingleNode("/titulkovac/font").Attributes.GetNamedItem("type").InnerText.ToString());
			xmlProperties.SelectSingleNode("/titulkovac/font").Attributes.GetNamedItem("size").InnerText = sFntSize;
			xmlProperties.SelectSingleNode("/titulkovac/message").Attributes.GetNamedItem("color-text").InnerText = sMsgColorText;
			xmlProperties.SelectSingleNode("/titulkovac/message").Attributes.GetNamedItem("color-background").InnerText = sMsgColorBackground;
			xmlProperties.SelectSingleNode("/titulkovac/message").Attributes.GetNamedItem("separator").InnerText = sMsgSeparator;
			xmlProperties.SelectSingleNode("/titulkovac/message").InnerText = sMsg;
			xmlProperties.SelectSingleNode("/titulkovac/application").Attributes.GetNamedItem("delay").InnerText = sDel;
			xmlProperties.SelectSingleNode("/titulkovac/application").Attributes.GetNamedItem("movement").InnerText = sMove;
			writeXMLProperties();
		}

		//vytvoreni noveho prazdneho XML souboru pro ulozeni vlastnosti aplikace
		private void createNewXmlFileProperties() {
			XmlElement xmlRoot = xmlProperties.CreateElement("titulkovac");
			xmlProperties.AppendChild(xmlRoot);
			XmlElement xmlFont = xmlProperties.CreateElement("font");
			XmlAttribute xmlFontTyp = xmlProperties.CreateAttribute("type");
			xmlFont.Attributes.Append(xmlFontTyp);
			XmlAttribute xmlFontSize = xmlProperties.CreateAttribute("size");
			xmlFont.Attributes.Append(xmlFontSize);
			xmlRoot.AppendChild(xmlFont);

			XmlElement xmlMessage = xmlProperties.CreateElement("message");
			XmlAttribute xmlMessColorText = xmlProperties.CreateAttribute("color-text");
			xmlMessage.Attributes.Append(xmlMessColorText);
			XmlAttribute xmlMessColorBackground = xmlProperties.CreateAttribute("color-background");
			xmlMessage.Attributes.Append(xmlMessColorBackground);
			XmlAttribute xmlMessSeparator = xmlProperties.CreateAttribute("separator");
			xmlMessage.Attributes.Append(xmlMessSeparator);
			xmlRoot.AppendChild(xmlMessage);

			XmlElement xmlApplication = xmlProperties.CreateElement("application");
			XmlAttribute xmlAppDelay = xmlProperties.CreateAttribute("delay");
			xmlApplication.Attributes.Append(xmlAppDelay);
			XmlAttribute xmlAppMovement = xmlProperties.CreateAttribute("movement");
			xmlApplication.Attributes.Append(xmlAppMovement);
			xmlRoot.AppendChild(xmlApplication);

			writeXMLProperties();
		}

		//ulozeni XML souboru s vlastnostmi aplikace
		private void writeXMLProperties() {
			xmlProperties.Save(System.IO.Path.Combine(Application.StartupPath, "Titulkovac.xml"));
		}

	}
}
